import { Button, ScrollView, Text, View, Swiper, SwiperItem} from '@ray-js/components';
import {
  getSystemInfo,
  onNetworkStatusChange,
  offNetworkStatusChange,
  onBluetoothAdapterStateChange,
  offBluetoothAdapterStateChange,
  showToast,
  showLoading,
  hideLoading,
  getAppInfo,
  bluetoothIsPowerOn,
  publishDps,
  getDeviceInfo,
  openDeviceDetailPage,
  openTimerPage,
} from '@ray-js/api';
import { hooks } from '@ray-js/panel-sdk';
import { router, usePageEvent } from 'ray';
import React from 'react';
import { useSelector } from '@/redux';
import styles from './index.module.less';
import { Image, Slider, Modal } from '@ray-js/components'
import iconTempr from '../../../public/icon-temperature.png'
import iconFanBg from '../../../public/icon-fan-bg.png'
import iconFan from '../../../public/icon-fan.png'
import iconOpen from '../../../public/icon-open.png'
import iconSleep from '../../../public/icon-sleep.png'
import iconLight from '../../../public/icon-light.png'
import iconVertical from '../../../public/icon-vertical.png'
import iconOpenOff from '../../../public/icon-open-off.png'
import middleware from '../../../public/icon-middleware.png'
import iconClose from '../../../public/icon-close.png'
import iconLightWhite from '../../../public/icon-light-white.png'
import iconSun from '../../../public/icon-sun.png'
import nature from '../../../public/icon-nature.png'
import natureOn from '../../../public/icon-nature-on.png'
import moon from '../../../public/icon-moon.png'
import moonOn from '../../../public/icon-moon-on.png'
import fresh from '../../../public/icon-fresh.png'
import freshOn  from '../../../public/icon-fresh-on.png'
import smart from '../../../public/icon-smart.png'
import smartOn from  '../../../public/icon-smart-on.png'
import hand from  '../../../public/icon-hand.png'
import handOn from  '../../../public/icon-hand-on.png'
import mode from '../../../public/icon-mode.png'
import modeOn from '../../../public/icon-mode-on.png'
import horizontalOff from '../../../public/icon-horizontal-off.png'
import horizontalOn from '../../../public/icon-horizontal-on.png'
import verticalOn from '../../../public/icon-vertical-on.png'
import verticalOff from '../../../public/icon-vertical-off.png'

import {useState, useEffect} from 'react'

const { useDevInfo, useDpState } = hooks;

export function Home() {
  const devInfo = useDevInfo();
  const devState = useDpState()
  const menu = [
    {
      title:'Switch',
      icon:iconOpen,
      offIcon: iconOpenOff
    },
    {
      title:'Sleep',
      icon:iconSleep
    },
    {
      title:'Light',
      icon:iconLight
    },
    {
      title:'Vertical',
      icon:iconVertical
    },
  ]

  const firstMenu = 
    [ {
      title:'Nature',
      icon:natureOn,
      offIcon: nature,
      id: 0
    }, {
      title:'Sleep',
      icon:moonOn,
      offIcon: moon,
      id: 1
    }, {
      title:'Fresh',
      icon:freshOn,
      offIcon: fresh,
      id: 2
    }, {
      title:'Smart',
      icon:smartOn,
      offIcon: smart,
      id: 3
    }]
   
  
  const secondMenu =  [
    {
      title:'Nature',
      icon:handOn,
      offIcon: hand,
      id: 4
    }, {
      title:'Mode',
      icon:modeOn,
      offIcon: mode,
      id: 5
    }
  ]

  const verticalMenu = [
    {
      title:'Vertical',
      icon:verticalOn,
      offIcon: verticalOff,
      id: 7,
      numId:[71, 72, 73]
    }, {
      title:'Horizontal',
      icon:horizontalOn,
      offIcon: horizontalOff,
      id:8,
      numId:[81, 82, 83]
    }
  ]

  const brightRange = [10, 1000]
  const cctRange = [0, 1000]

  const [currentTemp, setTemp] = useState(devState[0]['temp_current'] || 30)
  const [spinSpeed, setSpeed] = useState(devState[0]['fan_speed'] || 1)
  const [isOpen, setOpen] = useState(devState[0]['fan_switch'] || false)
  const [menuIndex, setMenuIndex] = useState(null)
  const [preTime, setPreTime] = useState(null)
  const [isShowModal, setModal] = useState(false)
  const [point, setPoint] = useState({pageX: getOrignPointX(), pageY: 234})
  const [selectColor, setColor] = useState(pointToRGB(point.pageX))
  //亮度
  const [bright, setBright] = useState(getValueInRange(devState[0]['bright_value'] || 10, brightRange).percent)
  const [isLightOn, setLightOn] = useState(devState[0]['switch_led'] ||  false)
  const [secondIndex, setSecondIndex] = useState(getFanMode())
  const [verticalId, setVerticalId] = useState(getOrignFanVertical())
  const [cctPercent, setCctPercent] = useState(getValueInRange(devState[0]['temp_value'] || 0, cctRange).percent)
  
  useEffect(() => {
    console.log('useEffffffffff', isOpen);
    uploadDps()
  }, [isOpen, spinSpeed, isLightOn, bright, cctPercent, secondIndex])

  function getFanMode() {
    const preMode = devState[0]['fan_mode']
    let selected = null
    for (let index = 0; index < firstMenu.length; index++) {
      const item = firstMenu[index];
      const title = item.title.toLowerCase()
      if (title == preMode) {
        selected = item.id
      }
    }
    return selected
  }
  
  function getOrignPointX() {
    const percent = getValueInRange(devState[0]['temp_value'] || 10, cctRange).percent
    return 28 + 319 * Number((percent / 100).toFixed())
  }
  //0-value 1-percent
  function getValueInRange(value, range, type = 0) {
    const diff = range[1] - range[0]
    const percent = type == 0 ? (value / diff) : (value / 100)
    return {
      value: type == 0 ? value : (range[0] + percent * diff).toFixed(),
      percent: (percent * 100).toFixed()
    }
  }

  function spinDuration() {
    return  2 - ((2 - 0.2) / 100) * spinSpeed + 's' 
  }

  function clickMenu(e) {
    console.log(e, preTime);
    // if ((e.timeStamp > preTime + 20) || !preTime) {
      setPreTime(e.timeStamp)
      const index = e.currentTarget.id
      if (index == 0) {
        setOpen(!isOpen)
        setMenuIndex(null)
       
      }else if (isOpen) {
        if (index == 2) {
          setMenuIndex(null)
          setModal(true)
        }else {
          if (index == menuIndex) {
            setMenuIndex( null)
          } else {
            setMenuIndex(index)
          }
        }
      // }
    }
  }

  function touchStart(e) {
      console.log(e, 'start touch');
    handlePoint(e)

  }

  function handleTouch(e) {
    // console.log(e, 'handle touch');
    // const point = e.changedTouches.length && e.changedTouches[0]
    // console.log(point, 'pppppppoint');
    // handlePoint(e)
  }

  function touchEnd(e) {
    console.log(e, 'end touch');
    handlePoint(e)
  }


  function handlePoint(e) {
    const {pageX, pageY} = e.changedTouches.length && e.changedTouches[0]
    let x = pageX
    let y = pageY
    if (pageX < 28) {
      x = 28
    }
    if (pageX > 347) {
      x = 347
    }
    if (pageY < 197) {
      y = 197
    }
    if (pageY > 320) {
      y =320
    }
    setPoint({pageX: x, pageY: y})
    const percent = ((x - 28) / 319 * 100).toFixed()
    setCctPercent(percent)
    setColor(pointToRGB(x))
  }

  function pointToRGB(pageX) {
    const half = (347 - 28) / 2
    const middle = 28 + half
    const leftColor = {h: 40, s:100, l:68}
    const rightColor = {h:202,s:96, l:90}
    let newL = leftColor.l
    let color = leftColor
    if (pageX < middle) {
      newL = 68 + (pageX - 28) * 32 / 161
      leftColor.l = newL
      color = leftColor
    } else {
      newL = 100 - (pageX - middle) * 10 / 161
      rightColor.l = newL
      color =  rightColor
    }
    // setLight(Number(newL).toFixed(0))
    console.log(newL, color, leftColor, rightColor);
    return HSLToRGB(color)
  }

  function HSLToRGB({h, s, l}) {
    s /= 100;
    l /= 100;
    const k = n => (n + h / 30) % 12;
    const a = s * Math.min(l, 1 - l);
    const f = n =>
      l - a * Math.max(-1, Math.min(k(n) - 3, Math.min(9 - k(n), 1)));
    const color = `rgb(${255 * f(0)},  ${255 * f(8)}, ${255 * f(4)})`
    console.log(color)
    return color
  };
  
   const handleLightOn = () => {
     setLightOn(!isLightOn)
   }

   function progressStart(e) {
     console.log(e, 'progressStart');
     handleProgress(e)
   }

   function progressMove(e) {
    console.log(e, 'progressMove');
    handleProgress(e)
   }

   function progressEnd(e) {
    console.log(e, 'progressEnd');
    handleProgress(e)
   }

   function handleProgress(e) {
    const {pageX, pageY} = e.changedTouches.length && e.changedTouches[0]
    const minX = 20
    const maxX = 355
    let differ = pageX - minX
    if (differ < 0) {
      differ = 0
    } else if (differ > 335) {
      differ = 335
    }
    const percent = (differ / 335 * 100).toFixed()
    setBright(percent)
     
   }

   function getOrignFanVertical() {
     const fan_vertical = devState[0]['fan_vertical']
     const fan_horizontal = devState[0]['fan_horizontal']
     const num_vertical = fan_vertical ? parseInt(fan_vertical / 30) : null
     const num_horizontal = fan_horizontal ? parseInt(fan_horizontal / 30) :null
     if (num_vertical) {
       return 70 + num_vertical
     }
     if (num_horizontal) {
       return 80 + num_horizontal
     }
     return null
   }

   //dpId 每个设备不一样
   function getSelectFanVertical() {
     if (verticalId) {
       const value = (verticalId % 70) * 30
      if (verticalId < 80) {
        return {
          '6' : value
        }
      }else {
        return {
          '7' : value
        }
      }
     }
     return {}
   }

   function uploadDps(params = {}) {
    const brightValue = getValueInRange(bright, brightRange, 1).value
    const cctValue = getValueInRange(cctPercent, cctRange, 1).value
    const mode = firstMenu.filter(item => item.id == secondIndex)
    const selectMode =  mode.length > 0 ?  {'61': mode[0].title.toLowerCase()} : {'61': null}
   
    console.log(brightValue, 'brightValue');
    console.log(cctValue, 'cctValue');
    publishDps({
      deviceId: devInfo.devId,
      // dps: { '60': isOpen, '61':selectMode,  '62': spinSpeed, '20':isLightOn, '22': brightValue, '23':cctValue}, // {'dpid': dpValue, '2': false}
      dps: { '60': isOpen, ...selectMode,  '62': spinSpeed, '20':isLightOn},
      mode: 2,
      pipelines: [],
      options: {}, // 0，静音； 1，震动；2,声音； 3，震动声音
      success: () => console.log('success'),
      fail: d => {
        console.log('-----返回结果错误?', d);
      },
    });
   }

   function handleSubmenu(id) {
     if (id == secondIndex) {
       setSecondIndex(null)
     } else {
       setSecondIndex(id)
     }
   }

   function submenuItem(index, arr = firstMenu, clickFun = handleSubmenu, currentIndex = secondIndex) {
    //  const arr = page == 0 ? firstMenu:secondMenu
     const {title, id, icon, offIcon} = arr[index]
     return (
      <View className={styles.button}>
        <Image className={styles.menuImage} src={id == currentIndex ? icon:offIcon} onClick={() => {clickFun(id)}}/>
        <Text className={styles.bTitle}>{title}</Text>
      </View>
     )
   }

   function handleVerticalMenu(id) {
     const tensPre = parseInt(verticalId/10)
    //  const tensCur = id / 10
     if (tensPre == id) {
       setVerticalId(null)
     } else {
       setVerticalId(id * 10 + 2)
     }
     
   }

   function verticalItem(index) {
     const item = verticalMenu[index]
     const textColor = (subIndex) => {
       return {color: verticalId == item.numId[subIndex] ? '#000000':'rgba(0,0,0,0.4)',
       width:'33.3%',
       textAlign:'center',
       lineHeight:'64px',
       fontWeight:'500'
     }}
     const clickNum = (index) => {
       const id = item.numId[index]
       if (id == verticalId) {
         setVerticalId(null)
       } else {
        setVerticalId(id)
       }
     }
     return (
       <View className={styles.verticalItem}>
         {submenuItem(index, verticalMenu, handleVerticalMenu, parseInt(verticalId / 10))}
         <View className={styles.verticalNumBg}>
           <View style={textColor(0)} onClick={() => {clickNum(0)}}>30</View>
           <View style={textColor(1)} onClick={() => {clickNum(1)}}>60</View>
           <View style={textColor(2)} onClick={() => {clickNum(2)}}>90</View>
         </View>
       </View>
     )
   }
 
  return (
     <View className={styles.page}>
        <View className={styles.header}>
          <View className={styles.alignCenter}>
          <Image className={styles.icon} src={iconTempr} />
          <View className={styles.title}>{currentTemp}°C</View>
          </View>
          {/* <View className={styles.timer}>Countdown to open 03:00</View> */}
        </View>
        <View className={styles.fanGroup}>
          <Image className={styles.fanBg} src={iconFanBg}></Image>
          <Image className={styles.fan} src={iconFan} style={{animationDuration: spinDuration(), animationPlayState: isOpen ? 'running' : 'paused'}}></Image>
        </View>
        <View className={styles.sliderBg}>
          <Slider
            activeColor='linear-gradient(180deg, #87ABF2 11.62%, #4B7EE2 100%)'
            blockColor='white'
            blockSize={28}
            min={0}
            max={100}
            value={spinSpeed}
            step={5}
            disabled={!isOpen}
            onChange={(e) => {
              console.log('SliderChange', e)
              setSpeed(e.value)
              // publishDps()
            }}
          />
          <View className={styles.fantBottom}>
            <View className={styles.fontBottomTint}>Fan Speed</View>
            <View className={styles.fontBottomTint}>{spinSpeed}</View>
          </View>
        </View>
        <View className={styles.flexRowBetween}>
            {
              menu.map(({title, icon, offIcon}, index) => {
                 return (
                  <View
                      className={styles.button}
                      key={index}
                    >
                      {
                      menuIndex == 1 ? 
                        <View className={styles.modalBg}>
                          <Swiper dots={true} dotActiveColor='#6395F6' dotColor='rgba(99,149,246, 0.2)'>
                            <SwiperItem>
                              <View className={styles.swiperItem}>
                                {
                                  submenuItem(0)
                                }
                                {
                                  submenuItem(1)
                                }
                                {
                                  submenuItem(2)
                                }
                                {
                                  submenuItem(3)
                                }
                              </View>
                            </SwiperItem>
                            {/* <SwiperItem>
                            <View className={styles.swiperItemNext}>
                              {
                                submenuItem(0, secondMenu)
                              }
                              {
                                submenuItem(1, secondMenu)
                              }
                              </View>
                            </SwiperItem> */}
                          </Swiper>
                        </View>
                      : menuIndex == 3 ? <View className={styles.modalBg} style={{height: '213px', display: 'flex', flexDirection:'column', padding:'16px 16px 0', gap:'12px'}}>
                       {
                         verticalItem(0)
                       }
                       {
                         verticalItem(1)
                       }
                      </View>
                      : null
                      }
                      <Image className={styles.middleware} src={middleware} style={{visibility: menuIndex == index ? 'visible':'hidden'}} />
                      <Image src={index > 0 ? icon: isOpen ? icon:offIcon} className={styles.menuImage} id={index}  style={{opacity: isOpen ? 1: index > 0 ? 0.4:1 }} onClick={
                        clickMenu
                      }></Image>
                      <Text className={styles.bTitle}>{title}</Text>
                  </View>
                 )
              })
            }
        </View>
      <View className={styles.modalback} style={{background:'rgba(0, 0, 0, 0.1)',backdropFilter:'blur(35px)', visibility: isShowModal ? 'visible':'hidden' }}>
      <View className={styles.lightModal}>
         <View className={styles.lightModalTop}>
           <View>Light
           </View>
           <Image className={styles.lightModalClose} src={iconClose} onClick={() => {
             setModal(false)
           }}></Image>
         </View>
          <View
            className={styles.lightMiddle}
            onTouchStart={touchStart}  
            onTouchMove={handleTouch}
            onTouchEnd={touchEnd}
          >
          </View>
          <View className={styles.progressBg} onTouchStart={progressStart}  
            onTouchMove={progressMove}
            onTouchEnd={progressEnd}>
            <Image className={styles.progressSun} src={iconSun}></Image>
            <View className={styles.progressTitle}>{bright}%</View>
            <View className={styles.progressBar} style={{width: bright + '%'}}></View>
          </View>
          <View className={styles.lightBtn} style={{backgroundColor: isLightOn ? '#6395F6':'rgba(0,0,0,0.5)'}} onClick={handleLightOn}>
            <Image className={styles.lightBtnIcon} src={iconLightWhite}></Image>
            <View className={styles.lightBtnTitle}>{isLightOn ? 'ON':'OFF'}</View>
            <View className={styles.lightBtnSubtitle}>{isLightOn ? '/OFF':'/ON'}</View>
          </View>
        </View>
        <View className={styles.moveCircle} style={{top: point.pageY - 14 + 'px',  left: point.pageX - 14 + 'px', backgroundColor: selectColor}}></View>
     </View>
     </View>
  )
}

export default Home;
